# coding: utf-8 or # -*- coding: utf-8 -*-
"""
如何将ELMo词向量用于中文  http://www.linzehui.me/2018/08/12/碎片知识点/如何将ELMo词向量用于中文/
通用词向量、句向量研究现状 https://www.jianshu.com/p/53de9d77e16c
代码来自：https://github.com/allenai/bilm-tf
"""